/***********************************************************************************************************************
 *
 * SolidBlue - open source safe data
 * Copyright (C) 2011-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://solidblue.java.net
 * SCM: https://bitbucket.org/tidalwave/solidblue-src
 *
 **********************************************************************************************************************/
package it.tidalwave.integritychecker.ui.impl;

import javax.annotation.Nonnull;
import javax.annotation.concurrent.NotThreadSafe;
import java.util.concurrent.TimeUnit;
import org.joda.time.Duration;
import org.joda.time.format.PeriodFormat;
import it.tidalwave.actor.MessageSupport;
import it.tidalwave.actor.Collaboration;
import it.tidalwave.actor.CollaborationCompletedMessage;
import it.tidalwave.actor.CollaborationStartedMessage;
import it.tidalwave.actor.annotation.Actor;
import it.tidalwave.actor.annotation.CollaborationListener;
import it.tidalwave.actor.annotation.Message;
import it.tidalwave.actor.annotation.MessageListener;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.integritychecker.FileDiscoveredMessage;
import it.tidalwave.integritychecker.FileScanRequestMessage;
import it.tidalwave.integritychecker.FingerprintComputedMessage;
import it.tidalwave.integritychecker.ui.IntegrityCheckerView;
import it.tidalwave.integritychecker.ui.IntegrityCheckerView.Data;
import it.tidalwave.integritychecker.ui.IntegrityCheckerViewController;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;

/***********************************************************************************************************************
 * 
 * @stereotype Actor
 * @stereotype Controller
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@Actor(threadSafe=false) @NotThreadSafe @Slf4j
public class DefaultIntegrityCheckerViewController implements IntegrityCheckerViewController
  {
    private static final double K10 = 1000;
    private static final double M10 = 1000000;
    
    @Message(outOfBand=true, daemon=true) @ToString
    private static class RefreshViewMessage extends MessageSupport
      {
      }
            
    private final IntegrityCheckerViewBuilder viewBuilder = Locator.find(IntegrityCheckerViewBuilder.class);
    
    private final IntegrityCheckerView view = (IntegrityCheckerView)(viewBuilder.createGui());
    
    private final Data data = new Data();
    
    private boolean refreshing = false;
    
    private int totalFileCount;
    
    private long totalDataSize;
    
    private int processedFileCount;
    
    private long processedDataSize;
    
    /*******************************************************************************************************************
     * 
     * Resets statistics when a new scan is started.
     * 
     ******************************************************************************************************************/
    @CollaborationListener(message=FileScanRequestMessage.class)
    public void scanStarted (final @Nonnull CollaborationStartedMessage message)
      {
        log.info("scanStarted({})", message);
        totalFileCount = 0;
        totalDataSize = 0;
        processedFileCount = 0;
        processedDataSize = 0;
        refreshing = true;
        new RefreshViewMessage().send();
      }
    
    /*******************************************************************************************************************
     * 
     * Stops activities when the scan has been completed.
     * 
     ******************************************************************************************************************/
    @CollaborationListener(message=FileScanRequestMessage.class)
    public void scanCompleted (final @Nonnull CollaborationCompletedMessage message)
      {
        log.info("scanCompleted({})", message);
        refreshing = false;
        data.setProgressLabel("done");
        new RefreshViewMessage().send();
      }
    
    /*******************************************************************************************************************
     * 
     * Updates the statistics for a new discovered file.
     * 
     ******************************************************************************************************************/
    @MessageListener
    public void updateStatistics (final @Nonnull FileDiscoveredMessage message)
      {
        log.trace("updateStatistics({})", message);
        totalFileCount++;
        totalDataSize += message.getFileObject().getSize();
      }
    
    /*******************************************************************************************************************
     * 
     * Updates the statistics for a new fingerprint computed.
     * 
     ******************************************************************************************************************/
    @MessageListener
    public void updateStatistics (final @Nonnull FingerprintComputedMessage message)
      {
        log.trace("updateStatistics({})", message);
        processedFileCount++;
        processedDataSize += message.getFileObject().getSize();
        final Collaboration collaboration = message.getCollaboration();
        final long elapsedTime = collaboration.getDuration().getMillis();
        final double speed = (processedDataSize / M10) / (elapsedTime / K10);
        final int eta = (int)(((totalDataSize - processedDataSize) / M10) / speed);
        
        data.setProcessed(String.format("%.1f MB in %d files", processedDataSize / M10, processedFileCount));
        data.setTotal(String.format("%.1f MB in %d files", totalDataSize / M10, totalFileCount));
        data.setElapsedTime(PeriodFormat.getDefault().print(new Duration(elapsedTime - elapsedTime % 1000).toPeriod()));
        data.setRemainingTime(PeriodFormat.getDefault().print(new Duration(eta * 1000).toPeriod()));
        data.setSpeed(String.format("%.1f MB/s", speed));
        data.setPending("" + collaboration.getDeliveringMessagesCount());
        data.setRunning("" + collaboration.getRunningThreadsCount());
        data.setProgress(processedDataSize / (float)totalDataSize);
        data.setProgressLabel(String.format("%.1f %%", 100 * data.getProgress()));
      }
    
    /*******************************************************************************************************************
     * 
     * Updates the view.
     * 
     ******************************************************************************************************************/
    @MessageListener
    private void updateView (final @Nonnull RefreshViewMessage message)
      {
        view.updateStatistics(data);
        
        if (refreshing)
          {
            new RefreshViewMessage().sendLater(1, TimeUnit.SECONDS);
          }
      }
  }
